"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_container_1 = require("@arkecosystem/core-container");
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const sequelize_1 = __importDefault(require("sequelize"));
const umzug_1 = __importDefault(require("umzug"));
class Database {
    /**
     * Set up the database connection.
     * @param  {Object} config
     * @return {void}
     */
    async setUp(config) {
        if (this.connection) {
            throw new Error("Webhooks database already initialised");
        }
        if (config.dialect === "sqlite" && config.storage !== ":memory:") {
            await fs_extra_1.default.ensureFile(config.storage);
        }
        this.connection = new sequelize_1.default(Object.assign({}, config, { operatorsAliases: sequelize_1.default.Op }));
        try {
            await this.connection.authenticate();
            await this.__runMigrations();
            this.__registerModels();
        }
        catch (error) {
            core_container_1.app.forceExit("Unable to connect to the database!", error);
        }
    }
    /**
     * Paginate all webhooks.
     * @param  {Object} params
     * @return {Object}
     */
    paginate(params) {
        return this.model.findAndCountAll(params);
    }
    /**
     * Get a webhook for the given id.
     * @param  {Number} id
     * @return {Object}
     */
    findById(id) {
        return this.model.findById(id);
    }
    /**
     * Get all webhooks for the given event.
     * @param  {String} event
     * @return {Array}
     */
    findByEvent(event) {
        return this.model.findAll({ where: { event } });
    }
    /**
     * Store a new webhook.
     * @param  {Object} data
     * @return {Object}
     */
    create(data) {
        return this.model.create(data);
    }
    /**
     * Update the webhook for the given id.
     * @param  {Number} id
     * @param  {Object} data
     * @return {Boolean}
     */
    async update(id, data) {
        try {
            const webhook = await this.model.findById(id);
            webhook.update(data);
            return true;
        }
        catch (e) {
            return false;
        }
    }
    /**
     * Destroy the webhook for the given id.
     * @param  {Number} id
     * @return {Boolean}
     */
    async destroy(id) {
        try {
            const webhook = await this.model.findById(id);
            webhook.destroy();
            return true;
        }
        catch (e) {
            return false;
        }
    }
    /**
     * Run all migrations.
     * @return {Boolean}
     */
    async __runMigrations() {
        const umzug = new umzug_1.default({
            storage: "sequelize",
            storageOptions: {
                sequelize: this.connection,
            },
            migrations: {
                params: [this.connection.getQueryInterface(), sequelize_1.default],
                path: path_1.default.join(__dirname, "migrations"),
            },
        });
        return umzug.up();
    }
    /**
     * Register all models.
     * @return {void}
     */
    __registerModels() {
        this.model = this.connection.define("webhook", {
            id: {
                allowNull: false,
                autoIncrement: true,
                primaryKey: true,
                type: sequelize_1.default.INTEGER,
            },
            event: sequelize_1.default.STRING,
            target: sequelize_1.default.STRING,
            conditions: sequelize_1.default.JSON,
            token: {
                unique: true,
                type: sequelize_1.default.STRING,
            },
            enabled: sequelize_1.default.BOOLEAN,
        }, {});
    }
}
exports.database = new Database();
//# sourceMappingURL=index.js.map