"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_container_1 = require("@arkecosystem/core-container");
const axios_1 = __importDefault(require("axios"));
const conditions = __importStar(require("./conditions"));
const database_1 = require("./database");
class WebhookManager {
    constructor() {
        this.logger = core_container_1.app.resolvePlugin("logger");
    }
    /**
     * Set up the webhook app.
     * @return {void}
     */
    async setUp() {
        const emitter = core_container_1.app.resolvePlugin("event-emitter");
        const blockchain = core_container_1.app.resolvePlugin("blockchain");
        for (const event of blockchain.getEvents()) {
            emitter.on(event, async (payload) => {
                const webhooks = await database_1.database.findByEvent(event);
                for (const webhook of this.getMatchingWebhooks(webhooks, payload)) {
                    try {
                        const response = await axios_1.default.post(webhook.target, {
                            timestamp: +new Date(),
                            data: payload,
                            event: webhook.event,
                        }, {
                            headers: {
                                Authorization: webhook.token,
                            },
                        });
                        this.logger.debug(`Webhooks Job ${webhook.id} completed! Event [${webhook.event}] has been transmitted to [${webhook.target}] with a status of [${response.status}].`);
                    }
                    catch (error) {
                        this.logger.error(`Webhooks Job ${webhook.id} failed: ${error.message}`);
                    }
                }
            });
        }
    }
    /**
     * Get all webhooks.
     * @param  {Array} webhooks
     * @param  {Object} payload
     * @return {Array}
     */
    getMatchingWebhooks(webhooks, payload) {
        const matches = [];
        for (const webhook of webhooks) {
            if (!webhook.enabled) {
                continue;
            }
            if (!webhook.conditions || (Array.isArray(webhook.conditions) && !webhook.conditions.length)) {
                matches.push(webhook);
                continue;
            }
            for (const condition of webhook.conditions) {
                const satisfies = conditions[condition.condition];
                if (!satisfies(payload[condition.key], condition.value)) {
                    continue;
                }
                matches.push(webhook);
            }
        }
        return matches;
    }
}
exports.webhookManager = new WebhookManager();
//# sourceMappingURL=manager.js.map